package com.ym.mec.auth.service.impl;

import com.ym.mec.auth.api.dto.SysUserInfo;
import com.ym.mec.auth.api.entity.LoginEntity;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.enums.YesOrNoEnum;
import com.ym.mec.auth.dal.dao.SysUserDao;
import com.ym.mec.auth.service.SysRoleMenuService;
import com.ym.mec.auth.service.SysUserRoleService;
import com.ym.mec.auth.service.SysUserService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImResult;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.im.UserFeignService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Service
public class SysUserServiceImpl extends BaseServiceImpl<Integer, SysUser> implements SysUserService {

	@Autowired
	private SysUserDao sysUserDao;

	@Autowired
	private SysUserRoleService sysUserRoleService;

	@Autowired
	private SysRoleMenuService sysRoleMenuService;
	@Autowired
	private ImFeignService imFeignService;
	@Autowired
	private UserFeignService userFeignService;
	@Autowired
	private SysUserService sysUserService;
	@Autowired
	private IdGeneratorService smsCodeService;

	@Value("${message.autoRegister}")
	private boolean autoRegister;
	@Value("${auth.sysconfig.tenantId}")
	private Integer lesseeOrganId;

	@Override
	public BaseDAO<Integer, SysUser> getDAO() {
		return sysUserDao;
	}

	@Override
	public SysUser queryByUsername(String username) {
		return sysUserDao.queryByUsername(username);
	}

	@Override
	public SysUser queryByPhone(String phone) {
		return sysUserDao.queryByPhone(phone);
	}

	@Override
	public SysUserInfo queryUserInfoByUsername(String username) {
		return getSysUserInfo(queryByUsername(username));
	}

	@Override
	public SysUserInfo queryUserInfoByPhone(String phone) {
		return getSysUserInfo(queryByPhone(phone));
	}

	private SysUserInfo getSysUserInfo(SysUser sysUser) {
		if (sysUser == null) {
			return null;
		}
		SysUserInfo userInfo = new SysUserInfo();
		userInfo.setSysUser(sysUser);

		List<String> roleCodeList = sysUserRoleService.queryRoleCodeListByUserId(sysUser.getId());
		userInfo.setRoles(roleCodeList.toArray(new String[roleCodeList.size()]));

		List<Integer> roleIdList = sysUserRoleService.queryRoleIdListByUserId(sysUser.getId());
		List<String> permissionList = sysRoleMenuService.queryPermissionsByRoleIdList(roleIdList);

		userInfo.setPermissions(permissionList.toArray(new String[permissionList.size()]));

		return userInfo;
	}

	@Override
	public boolean deleteById(Integer id) {
		SysUser user = get(id);

		if (user == null) {
			throw new BizException("账户不存在");
		}

		user.setUpdateTime(new Date());
		user.setDelFlag(YesOrNoEnum.YES);

		update(user);

		return true;
	}

	@Override
	public void updatePassword(String mobile, String password) {
		sysUserDao.updatePassword(mobile,password);
	}

	@Override
	public Integer add(SysUser sysUser) {
		sysUserDao.insert(sysUser);
		ImResult imResult = imFeignService.register(new ImUserModel(sysUser.getId().toString(), sysUser.getUsername(), sysUser.getAvatar()));
		sysUser.setImToken(imResult.getToken());
		sysUserDao.update(sysUser);
		return sysUser.getId();
	}

	@Override
	public ImUserModel getBasic(Integer userId) {
		return sysUserDao.getBasic(userId);
	}

	@Override
	public void updateBaseInfo(SysUser sysUser) {
		sysUserDao.update(sysUser);
		imFeignService.update(new ImUserModel(sysUser.getId().toString(),sysUser.getRealName()==null?sysUser.getUsername():sysUser.getRealName(),sysUser.getAvatar()));
	}

	@Override
	public SysUser queryUserInfo(Integer userId) {
		return sysUserDao.get(userId);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public SysUserInfo initUser(String phone, String clientId,String isRegister,String isLessee) {
		if("register".equals(isRegister)){
			throw new UsernameNotFoundException("404.9");
		}
		if(StringUtils.isNotEmpty(isLessee) || autoRegister){
			if(StringUtils.isNotEmpty(isLessee) && StringUtils.equalsIgnoreCase(clientId,"TEACHER")){
				SysUser sysUser = new SysUser();
				sysUser.setPhone(phone);
				sysUser.setUserType("TEACHER");
				sysUserDao.insert(sysUser);
				//添加用户现金账户
				imFeignService.register(new ImUserModel(sysUser.getId().toString(),phone,null));
				userFeignService.createCashAccount(sysUser.getId());
				//sysTenantAccount
				sysUserDao.insertSysTenantAccount(sysUser.getId());
				//创建teacher表
				sysUserDao.insertTeacher(sysUser.getId(),lesseeOrganId);
				return queryUserInfoByPhone(phone);
			}else if(StringUtils.isNotEmpty(isLessee) && StringUtils.equalsIgnoreCase(clientId,"STUDENT")){
				SysUser sysUser = new SysUser();
				sysUser.setPhone(phone);
				sysUser.setUserType("STUDENT");
				sysUser.setOrganId(lesseeOrganId);
				sysUserDao.insert(sysUser);
				sysUserService.saveStudent(sysUser.getId());
				//添加用户现金账户
				imFeignService.register(new ImUserModel(sysUser.getId().toString(),phone,null));
				userFeignService.createCashAccount(sysUser.getId());
				return queryUserInfoByPhone(phone);
			}
		}
		throw new UsernameNotFoundException("404.9");
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void retrieveUser(String phone, Authentication authentication) {
		LoginEntity loginEntity = (LoginEntity) authentication.getCredentials();
		SysUserInfo userInfo = sysUserService.queryUserInfoByPhone(loginEntity.getPhone());
		if (userInfo == null) {
			userInfo = sysUserService.initUser(loginEntity.getPhone(),loginEntity.getClientId(),loginEntity.getIsRegister(),loginEntity.getIsLessee());
		}else if(StringUtils.isNotEmpty(loginEntity.getIsLessee())){
			throw new LockedException("用户已存在");
		}
		if("EDUCATION".equals(loginEntity.getClientId())){
			loginEntity.setClientId("SYSTEM");
		}
		if (!userInfo.getSysUser().getUserType().contains(loginEntity.getClientId())) {
			throw new LockedException("用户不存在,请联系教务老师");
		}
//			 验证码验证
		boolean b = smsCodeService.verifyValidCode(loginEntity.getPhone(), loginEntity.getSmsCode());
		if(!b) throw new BadCredentialsException("验证码校验失败");
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void refreshImToken(SysUser sysUser) {
		sysUserDao.refreshImToken(sysUser.getId(),sysUser.getImToken());
	}

	@Override
	public void saveStudent(Integer userId) {
		sysUserDao.saveStudent(userId);
	}

}
