package com.ym.mec.biz.dal.dao;

import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dto.EmployeeDto;
import com.ym.mec.biz.dal.dto.SimpleUserDto;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.common.dal.BaseDAO;

import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

public interface EmployeeDao extends BaseDAO<Integer, Employee> {

    /**
     * @Author: Joburgess
     * @Date: 2019/9/17
     * 根据部门获取下面的员工
     */
    List<EmployeeDto> queryEmployByOrganId(Map<String,Object> params);

    /**
     * @Author: Joburgess
     * @Date: 2019/9/17
     * 部门员工统计
     */
    int queryEmployByOrganIdCount(Map<String,Object> params);

    /**
     * 修改密码
     * @param userID
     * @param password
     */
    void updatePassword(@Param("userID") Long userID, @Param("password") String password);

    /**
     * @Author: Joburgess
     * @Date: 2019/9/17
     * 修改用户账户锁定状态
     */
    int updateUserLockStatus(@Param("userID") Long userID);

    /**
     * @Author: Joburgess
     * @Date: 2019/9/25
     * 修改用户离职日期
     */
    int updateUserDemissionDate(@Param("userID") Long userID);
    /**
     * 根据手机号查询对象
     * @param phone
     * @return
     */
    SysUser queryByPhone(String phone);

    /**
     * 删除用户角色
     * @param userId
     */
    void delEmployeeRole(Integer userId);

    /**
     * 批量新增用户角色
     * @param id
     * @param roleIds
     */
    void batchAddEmployeeRole(@Param("userId") Integer id, @Param("roleIds")List<Integer> roleIds);

    /**
     * 获取用户权限列表
     * @param userId
     * @return
     */
    List<Integer> queryUserRole(Integer userId);

    /**
     * @describe 根据角色和用户查找用户
     * @author Joburgess
     * @date 2020/3/12
     * @param roleIds: 角色编号列表
     * @param organIds: 部门编号列表
     * @return java.util.List<com.ym.mec.biz.dal.dto.SimpleUserDto>
     */
    List<SimpleUserDto> findByRole(@Param("roleIds") String roleIds,
                                   @Param("organIds") String organIds);

    /**
     * @describe 根据用户编号获取用户
     * @author Joburgess
     * @date 2020/3/12
     * @param userIds:
     * @return java.util.List<com.ym.mec.biz.dal.dto.SimpleUserDto>
     */
    List<SimpleUserDto> findByIds(@Param("userIds") List<Integer> userIds);
}
