package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.CourseGroupExportDto;
import com.ym.mec.biz.dal.dto.PracticeCourseDto;
import com.ym.mec.biz.dal.dto.PracticeGroupDto;
import com.ym.mec.biz.dal.dto.StudentBuyPracticeDto;
import com.ym.mec.biz.dal.entity.ClassGroup;
import com.ym.mec.biz.dal.entity.CourseScheduleEvaluate;
import com.ym.mec.biz.dal.entity.PracticeGroup;
import com.ym.mec.biz.dal.enums.GroupStatusEnum;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;

public interface PracticeGroupDao extends com.ym.mec.common.dal.BaseDAO<Long, PracticeGroup> {

    /**
     * @param userId:    用户编号
     * @param startDate: 开始时间
     * @param endDate:   结束时间
     * @return java.util.List<com.ym.mec.biz.dal.entity.PracticeGroup>
     * @describe 获取学生指定日期内预约的陪练课
     * @author Joburgess
     * @date 2020/1/31
     */
    List<PracticeGroup> getUserPracticeCoursesWithDateRange(@Param("userId") Integer userId,
                                                            @Param("startDate") Date startDate,
                                                            @Param("endDate") Date endDate);

    /**
     * @param userId:
     * @return com.ym.mec.biz.dal.entity.PracticeGroup
     * @describe 获取学生的免费陪练课
     * @author Joburgess
     * @date 2020/2/24
     */
    PracticeGroup getUserFreePracticeGroup(@Param("userId") Integer userId);

    /**
     * @param groups:
     * @return int
     * @describe 批量更新
     * @author Joburgess
     * @date 2020/2/23
     */
    int batchUpdate(@Param("groups") List<PracticeGroup> groups);

    /**
     * @param userId:
     * @return java.util.List<com.ym.mec.biz.dal.entity.PracticeGroup>
     * @describe 统计用户的申请记录
     * @author Joburgess
     * @date 2020/2/3
     */
    int countUserPracticeApplyRecord(@Param("userId") Integer userId);

    /**
     * count陪练课列表数量
     *
     * @param params
     * @return
     */
    int countPracticeGroupByOrgan(Map<String, Object> params);

    /**
     * 获取陪练课列表
     *
     * @param params
     * @return
     */
    List<PracticeGroupDto> findAllByOrgan(Map<String, Object> params);

    /**
     * 获取已上课程数量
     *
     * @param practiceGroupIds
     * @return
     */
    List<Map<Long, Integer>> countPracticeGroupOverCourse(List<String> practiceGroupIds);

    /**
     * 修改老师
     *
     * @param practiceGroupId
     * @param teacherId
     */
    void updateUserId(@Param("practiceGroupId") String practiceGroupId, @Param("teacherId") Integer teacherId);


    /**
     * @param userId:
     * @param groupId:
     * @return com.ym.mec.biz.dal.entity.PracticeGroup
     * @describe 获取学生指定的课程组
     * @author Joburgess
     * @date 2020/2/19
     */
    PracticeGroup findUserPracticeGroup(@Param("userId") Integer userId,
                                        @Param("groupId") Long groupId);

    /**
     * @param userId:
     * @param groupId:
     * @return com.ym.mec.biz.dal.entity.PracticeGroup
     * @describe 获取学生指定的课程组
     * @author Joburgess
     * @date 2020/2/19
     */
    PracticeGroupDto findUserPracticeGroup2(@Param("userId") Integer userId,
                                            @Param("groupId") Long groupId);

    /**
     * @param userId: 学生编号
     * @return com.ym.mec.biz.dal.entity.PracticeGroup
     * @describe 获取学生最后一次购买的陪练课组
     * @author Joburgess
     * @date 2020/2/17
     */
    PracticeGroup findUserLatestPracticeGroup(@Param("userId") Integer userId);

    /**
     * @param userId: 用户编号
     * @return java.util.List<com.ym.mec.biz.dal.dto.PracticeCourseDto>
     * @describe 获取用户购买的陪练课
     * @author Joburgess
     * @date 2020/2/18
     */
    List<PracticeCourseDto> findUserBuyPracticeGroups(@Param("userId") Integer userId);


    List<PracticeCourseDto> findUserBuyPracticeGroupsWithDate(@Param("userId") Integer userId,
                                                              @Param("date") Date date);

    /**
     * @return java.util.List<com.ym.mec.biz.dal.entity.PracticeGroup>
     * @describe 获取已经达到截至时间的陪练课程组
     * @author Joburgess
     * @date 2020/2/23
     */
    List<PracticeGroup> findHistoryPracticeGroups();

    /**
     * @param groupId:
     * @return com.ym.mec.biz.dal.entity.PracticeGroup
     * @describe
     * @author Joburgess
     * @date 2020/2/24
     */
    PracticeGroup lockPracticeGroup(@Param("groupId") Integer groupId);

    /**
     * @param userId:
     * @return java.util.List<com.ym.mec.biz.dal.entity.PracticeGroup>
     * @describe 获取用户指定状态的陪练课程组
     * @author Joburgess
     * @date 2020/2/26
     */
    List<PracticeGroup> findUserStatusPracticeGroups(@Param("userId") Integer userId,
                                                     @Param("groupStatus") GroupStatusEnum groupStatusEnum);

    /**
     * @param userId:
     * @param date:
     * @return java.util.List<com.ym.mec.biz.dal.entity.PracticeGroup>
     * @describe 获取用户指定日期的未成功订单
     * @author Joburgess
     * @date 2020/3/2
     */
    List<PracticeGroup> findUserLockPracticeGroupWithDate(@Param("userId") Integer userId,
                                                          @Param("date") Date date);

    /**
     * @param userId:
     * @param date:
     * @return java.util.List<com.ym.mec.biz.dal.entity.PracticeGroup>
     * @describe 获取用户指定日期及之前的未成功订单
     * @author Joburgess
     * @date 2020/3/2
     */
    List<PracticeGroup> findUserLockPracticeGroupWithDateBefore(@Param("userId") Integer userId,
                                                                @Param("date") Date date);

    List<PracticeGroupDto> findPracticeGroupsReviews(Map<String, Object> params);

    Integer countPracticeGroupReviews(Map<String, Object> params);

    /**
     * 獲取要预生成报告的数据
     *
     * @param nowDate
     * @param afterDate
     * @return
     */
    List<CourseScheduleEvaluate> getNeedPostReportPracticeGroups(@Param("nowDate") Date nowDate, @Param("afterDate") Date afterDate, @Param("afterDateIsLastDay") boolean afterDateIsLastDay);

    /**
     * 获取报告的课次数
     *
     * @param classGroupIds
     * @param startDate
     * @param endDate
     * @return
     */
    List<ClassGroup> getReportCourseTimes(@Param("classGroupIds") List<Integer> classGroupIds, @Param("startDate") Date startDate, @Param("endDate") Date endDate);

    /**
     * 根据课程组id查询课程组详情
     *
     * @param id
     * @return
     */
    PracticeCourseDto findByGroupId(@Param("id") Integer id);

    /**
     * COUNT网管课购买列表
     *
     * @param params
     * @return
     */
    int countStudentBuys(Map<String, Object> params);

    /**
     * query网管课购买列表
     *
     * @param params
     * @return
     */
    List<StudentBuyPracticeDto> queryStudentBuys(Map<String, Object> params);


    /**
     * 获取导出网管课信息
     * @param organIds
     * @return
     */
    List<CourseGroupExportDto> getPracticeGroupExport(@Param("organIds") String organIds);
}
