package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.ExtracurricularExercisesDao;
import com.ym.mec.biz.dal.dao.ExtracurricularExercisesReplyDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.entity.ExtracurricularExercises;
import com.ym.mec.biz.dal.entity.ExtracurricularExercisesReply;
import com.ym.mec.biz.dal.entity.Teacher;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.page.ExtraExercilseQueryInfo;
import com.ym.mec.biz.service.ExtracurricularExercisesService;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ExtracurricularExercisesServiceImpl extends BaseServiceImpl<Long, ExtracurricularExercises> implements ExtracurricularExercisesService {
	
	@Autowired
	private ExtracurricularExercisesDao extracurricularExercisesDao;
	@Autowired
	private ExtracurricularExercisesReplyDao extracurricularExercisesReplyDao;
	@Autowired
	private SysMessageService sysMessageService;
	@Autowired
	private TeacherDao teacherDao;

	@Override
	public BaseDAO<Long, ExtracurricularExercises> getDAO() {
		return extracurricularExercisesDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
	public void createExtraExercises(ExtracurricularExercises exercises) {
		if(StringUtils.isBlank(exercises.getTitle())){
			throw new BizException("请填写标题");
		}
		if(StringUtils.isBlank(exercises.getContent())){
			throw new BizException("请填写内容");
		}
		if(StringUtils.isBlank(exercises.getStudentIdList())){
			throw new BizException("请指定学生");
		}
		if(Objects.isNull(exercises.getExpireDate())){
			LocalDateTime localDateTime=LocalDateTime.now();
			localDateTime=localDateTime.plusDays(2).withHour(21).withMinute(0).withSecond(0);
			exercises.setExpireDate(Date.from(localDateTime.atZone(DateUtil.zoneId).toInstant()));
		}
		List<Integer> studentIds = Arrays.asList(exercises.getStudentIdList().split(",")).stream().mapToInt(Integer::valueOf).boxed().collect(Collectors.toList());
		exercises.setBatchNo(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")));
		exercises.setExpectNum(studentIds.size());
		extracurricularExercisesDao.insert(exercises);
		String dateStr = DateUtil.dateToString(exercises.getExpireDate(), "MM月dd日HH点");
		Teacher teacher = teacherDao.get(exercises.getTeacherId());
		for (Integer studentId : studentIds) {
			ExtracurricularExercisesReply studentExtraExercise=new ExtracurricularExercisesReply();
			studentExtraExercise.setExtracurricularExercisesId(exercises.getId());
			studentExtraExercise.setUserId(studentId);
			studentExtraExercise.setStatus(0);
			studentExtraExercise.setIsReplied(0);
			studentExtraExercise.setIsView(0);
			studentExtraExercise.setIsRepliedTimely(0);
			extracurricularExercisesReplyDao.insert(studentExtraExercise);

			String notifyUrl = "3?courseScheduleID=" + studentExtraExercise.getId() + "&studentCourseHomeworkId=" + studentExtraExercise.getId() + "&extra=1";
			sysMessageService.batchSendImMessage(MessageTypeEnum.EXTRA_REMIND_IM,exercises.getTeacherId().toString(),null,
					new String[]{studentId.toString()},
					null, exercises.getTitle(), dateStr,exercises.getContent());

			Map<Integer, String> userMap = new HashMap<>();
			userMap.put(studentId, studentId.toString());
			sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.EXTRA_REMIND_PUSH,
					userMap, null, 0, notifyUrl, "STUDENT",
					teacher.getRealName(),exercises.getTitle(), dateStr);
		}
	}

	@Override
	public List<Map<String, Object>> findExtraExercilses(ExtraExercilseQueryInfo queryInfo) {
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		List<Map<String, Object>> result = new ArrayList<>();

		List<ExtracurricularExercises> dataList = extracurricularExercisesDao.findExtraExercises(params);
		if(!CollectionUtils.isEmpty(dataList)){
			List<Long> extraExerciseIds = dataList.stream().map(ExtracurricularExercises::getId).collect(Collectors.toList());
			List<ExtracurricularExercisesReply> extraExerciseStudentsByExtraExercises = extracurricularExercisesReplyDao.findExtraExerciseStudentsByExtraExercises(extraExerciseIds);
			Map<Long, List<ExtracurricularExercisesReply>> idStudentsMap = extraExerciseStudentsByExtraExercises.stream().collect(Collectors.groupingBy(ExtracurricularExercisesReply::getExtracurricularExercisesId));
			for (ExtracurricularExercises extracurricularExercises : dataList) {
				List<ExtracurricularExercisesReply> extracurricularExercisesReplies = idStudentsMap.get(extracurricularExercises.getId());
				if(CollectionUtils.isEmpty(extracurricularExercisesReplies)){
					continue;
				}
				long noRepliedNum = extracurricularExercisesReplies.stream().filter(e -> e.getStatus()==1&&e.getIsReplied() == 0).count();
				if(noRepliedNum<=0){
					extracurricularExercises.setIsReplied(1);
				}
				long notSubmitNum = extracurricularExercisesReplies.stream().filter(e -> e.getStatus() == 0).count();
				if(notSubmitNum<=0){
					extracurricularExercises.setIsSubmit(1);
				}
				if(noRepliedNum>0){
					extracurricularExercises.setStatus(1);
				}else if(noRepliedNum<=0&&notSubmitNum>0){
					extracurricularExercises.setStatus(2);
				}else if(noRepliedNum<=0&&notSubmitNum<=0){
					extracurricularExercises.setStatus(3);
				}
			}

			Map<String, List<ExtracurricularExercises>> collect = dataList.stream().collect(Collectors.groupingBy(e -> DateUtil.dateToString(e.getCreateTime(), "yyyy-MM-dd")));
			List<Date> dates=new ArrayList<>();
			collect.keySet().forEach(ds-> dates.add(DateUtil.stringToDate(ds,"yyyy-MM-dd")));
			dates.sort(Comparator.comparing(Date::getTime));
			dates.sort(Comparator.reverseOrder());
			for (Date date : dates) {
				String key=DateUtil.dateToString(date, "yyyy-MM-dd");
				Map<String, Object> r = new HashMap<>();
				r.put("day", key);
				collect.get(key).sort(Comparator.comparing(ExtracurricularExercises::getCreateTime).reversed());
				r.put("list", collect.get(key));
				result.add(r);
			}
		}
		return result;
	}
}
