package com.ym.mec.biz.service.impl;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.OrganizationDao;
import com.ym.mec.biz.dal.dao.SporadicChargeInfoDao;
import com.ym.mec.biz.dal.dao.StudentPaymentOrderDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.entity.SporadicChargeInfo;
import com.ym.mec.biz.dal.entity.StudentPaymentOrder;
import com.ym.mec.biz.dal.entity.SysUserCashAccount;
import com.ym.mec.biz.dal.entity.SysUserCashAccountDetail;
import com.ym.mec.biz.dal.enums.DealStatusEnum;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.PlatformCashAccountDetailTypeEnum;
import com.ym.mec.biz.dal.page.SporadicChargeInfoQueryInfo;
import com.ym.mec.biz.service.SporadicChargeInfoService;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.biz.service.SysUserCashAccountDetailService;
import com.ym.mec.biz.service.SysUserCashAccountService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class SporadicChargeInfoImpl extends BaseServiceImpl<Integer, SporadicChargeInfo> implements SporadicChargeInfoService {

    @Autowired
    private SporadicChargeInfoDao sporadicChargeInfoDao;
    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    private OrganizationDao organizationDao;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private SysUserCashAccountService sysUserCashAccountService;
    @Autowired
    private SysUserCashAccountDetailService sysUserCashAccountDetailService;

    @Override
    public BaseDAO<Integer, SporadicChargeInfo> getDAO() {
        return sporadicChargeInfoDao;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public Boolean renewForCallback(StudentPaymentOrder studentPaymentOrder) {
        int updateCount = studentPaymentOrderDao.update(studentPaymentOrder);
        if (updateCount <= 0) {
            throw new BizException("订单更新失败");
        }
        Integer userId = studentPaymentOrder.getUserId();
        SysUser sysUser = sysUserFeignService.queryUserById(userId);
        SporadicChargeInfo info = sporadicChargeInfoDao.get(Integer.parseInt(studentPaymentOrder.getMusicGroupId()));
        Map<Integer, String> map = new HashMap();
        map.put(userId, sysUser.getPhone());
        Map<Integer, String> yimei = new HashMap();
        map.put(userId, userId.toString());
        if (studentPaymentOrder.getStatus() == DealStatusEnum.SUCCESS) {
            //福袋活动，增加课程余额
            if (info.getChargeType().getCode() == 6) {
                sysUserCashAccountService.appendCourseBalance(userId, studentPaymentOrder.getActualAmount(), "活动");
            }

            //零星收费账户充值
            if (info.getChargeType().getCode() == 9) {
                sysUserCashAccountService.updateBalance(userId, studentPaymentOrder.getActualAmount(),PlatformCashAccountDetailTypeEnum.RECHARGE, "零星收费账户充值");
            }

            //插入交易明细
            BigDecimal amount = studentPaymentOrder.getActualAmount();
            Date nowDate = new Date();
            SysUserCashAccount cashAccount = sysUserCashAccountService.get(userId);
            //充值
            SysUserCashAccountDetail rechargeDetail = new SysUserCashAccountDetail();
            rechargeDetail.setAmount(amount);
            rechargeDetail.setBalance(cashAccount.getBalance().add(amount));
            rechargeDetail.setComment("缴费前充值");
            rechargeDetail.setCreateTime(nowDate);
            rechargeDetail.setStatus(DealStatusEnum.SUCCESS);
            rechargeDetail.setTransNo(studentPaymentOrder.getTransNo());
            rechargeDetail.setType(PlatformCashAccountDetailTypeEnum.RECHARGE);
            rechargeDetail.setUpdateTime(nowDate);
            rechargeDetail.setUserId(userId);
            rechargeDetail.setChannel(studentPaymentOrder.getPaymentChannel());
            rechargeDetail.setComAmount(studentPaymentOrder.getComAmount());
            rechargeDetail.setPerAmount(studentPaymentOrder.getPerAmount());
            sysUserCashAccountDetailService.insert(rechargeDetail);

            //缴费
            SysUserCashAccountDetail paymentDetail = new SysUserCashAccountDetail();
            paymentDetail.setAmount(amount.negate());
            paymentDetail.setBalance(cashAccount.getBalance());
            paymentDetail.setComment(info.getTitle());
            paymentDetail.setCreateTime(nowDate);
            paymentDetail.setStatus(DealStatusEnum.SUCCESS);
            paymentDetail.setTransNo(studentPaymentOrder.getTransNo());
            paymentDetail.setType(PlatformCashAccountDetailTypeEnum.PAY_FEE);
            paymentDetail.setUpdateTime(nowDate);
            paymentDetail.setUserId(userId);
            rechargeDetail.setChannel(studentPaymentOrder.getPaymentChannel());
            if (studentPaymentOrder.getComAmount() != null) {
                rechargeDetail.setComAmount(studentPaymentOrder.getComAmount().negate());
                rechargeDetail.setPerAmount(studentPaymentOrder.getPerAmount().negate());
            }

            sysUserCashAccountDetailService.insert(paymentDetail);

            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.SMS_SPORADIC_PAYMENT_SUCCESS, map, null, 0, "1","STUDENT",
                    studentPaymentOrder.getActualAmount(), info.getTitle());
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI, MessageTypeEnum.SMS_SPORADIC_PAYMENT_SUCCESS, yimei, null, 0, "1","",
                    studentPaymentOrder.getActualAmount(), info.getTitle());
            return true;
        }
        if (studentPaymentOrder.getStatus() == DealStatusEnum.CLOSE || studentPaymentOrder.getStatus() == DealStatusEnum.FAILED) {
            if (studentPaymentOrder.getBalancePaymentAmount() != null && studentPaymentOrder.getBalancePaymentAmount().compareTo(BigDecimal.ZERO) > 0) {
                sysUserCashAccountService.updateBalance(studentPaymentOrder.getUserId(), studentPaymentOrder.getBalancePaymentAmount(), PlatformCashAccountDetailTypeEnum.REFUNDS, info.getTitle() + "支付失败");
            }

            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.SMS_SPORADIC_PAYMENT_FAILED, map, null, 0, "","STUDENT",
                    studentPaymentOrder.getActualAmount(), info.getTitle());
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI, MessageTypeEnum.SMS_SPORADIC_PAYMENT_FAILED, yimei, null, 0, "", "",
                    studentPaymentOrder.getActualAmount(), info.getTitle());
            return false;
        }
        return false;
    }

    @Override
    public PageInfo<SporadicChargeInfo> queryDetailPage(SporadicChargeInfoQueryInfo queryInfo) {
        PageInfo<SporadicChargeInfo> pageInfo = queryPage(queryInfo);
        List<SporadicChargeInfo> rows = pageInfo.getRows();
        if (rows != null && rows.size() > 0) {
            if(queryInfo.getIsExport() && rows.size() > 50000){
                throw new BizException("数据集太大，不能导出.最大数据集不能超过50000");
            }
            Set<Integer> organIds = rows.stream().map(e -> e.getOrganId()).collect(Collectors.toSet());
            List<Integer> userIds = rows.stream().map(e -> e.getUserId()).collect(Collectors.toList());
            if(userIds == null){
                userIds = new ArrayList<>();
            }
            Set<Integer> operatorIds = rows.stream().map(e -> e.getOperatorId()).collect(Collectors.toSet());
            Map<Integer, String> organNames = MapUtil.convertMybatisMap(organizationDao.findOrganNameMapList(organIds));
            Map<Integer, String> operatorNames = MapUtil.convertMybatisMap(teacherDao.queryNameByIds(StringUtils.join(operatorIds, ",")));
            Map<Integer, String> userNames = MapUtil.convertMybatisMap(teacherDao.queryUsernameByIds(userIds));
            rows.forEach(e -> {
                e.setOrganName(organNames.get(e.getOrganId()));
                if (e.getUserId() != null) {
                    e.setUserName(userNames.get(e.getUserId()));
                }
                e.setOperatorName(operatorNames.get(e.getOperatorId()));
            });
        }
        return pageInfo;
    }

    @Override
    public SporadicChargeInfo getDetail(Integer id) {
        SporadicChargeInfo sporadicChargeInfo = sporadicChargeInfoDao.get(id);
        if (sporadicChargeInfo.getUserId() != null) {
            SysUser sysUser = sysUserFeignService.queryUserById(sporadicChargeInfo.getUserId());
            sporadicChargeInfo.setUserName(sysUser.getUsername());
        }
        return sporadicChargeInfo;
    }

    @Override
    public SporadicChargeInfo findByOrganIdAndType(Integer organId, Integer type) {
        return sporadicChargeInfoDao.findByOrganIdAndType(organId, type);
    }
}
