package com.ym.mec.biz.service.impl;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.CourseHomeworkStudentDetailDto;
import com.ym.mec.biz.dal.dto.SimpleUserDto;
import com.ym.mec.biz.dal.dto.StudentCourseHomeworkDto;
import com.ym.mec.biz.dal.dto.StudentHomeworkRecordDto;
import com.ym.mec.biz.dal.entity.CourseHomework;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.CourseScheduleReview;
import com.ym.mec.biz.dal.entity.StudentCourseHomework;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.page.CourseHomeworkQueryInfo;
import com.ym.mec.biz.service.CourseHomeworkService;
import com.ym.mec.biz.service.StudentCourseHomeworkService;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class StudentCourseHomeworkServiceImpl extends BaseServiceImpl<Long, StudentCourseHomework> implements StudentCourseHomeworkService {

    @Autowired
    private StudentCourseHomeworkDao studentCourseHomeworkDao;
    @Autowired
    private CourseHomeworkService courseHomeworkService;
    @Autowired
    private CourseHomeworkDao courseHomeworkDao;
    @Autowired
    private CourseScheduleDao courseScheduleDao;
    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    private CourseScheduleReviewDao courseScheduleReviewDao;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private SysConfigDao sysConfigDao;

    @Override
    public BaseDAO<Long, StudentCourseHomework> getDAO() {
        return studentCourseHomeworkDao;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public long insert(StudentCourseHomework bean) {
        SysUser user = sysUserFeignService.queryUserInfo();
        bean.setUserId(user.getId());
        bean.setStatus(YesOrNoEnum.YES);
        long insert = super.insert(bean);

        CourseSchedule courseSchedule = courseScheduleDao.get(bean.getCourseScheduleId());

        CourseHomework courseHomework = new CourseHomework();
        courseHomework.setId(bean.getCourseHomeworkId());
        courseHomework.setGroupType(courseSchedule.getGroupType());
        courseHomework.setCompletedNum(studentCourseHomeworkDao.countCompletedStudentNum(bean.getCourseHomeworkId()));

        courseHomeworkService.update(courseHomework);
        return insert;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void submitHomework(StudentCourseHomework bean) {
        if(Objects.isNull(bean.getAttachments())||StringUtils.isBlank(bean.getAttachments())){
            throw new BizException("请选择作业");
        }
        StudentCourseHomework existHomework = studentCourseHomeworkDao.get(bean.getId());
        if (Objects.isNull(bean) || Objects.isNull(bean.getId()) || Objects.isNull(existHomework)) {
            throw new BizException("作业不存在");
        }
        boolean push = Objects.isNull(existHomework.getAttachments());
        bean.setSubmitTime(new Date());
        super.update(bean);
        bean = studentCourseHomeworkDao.get(bean.getId());
        CourseSchedule courseSchedule = courseScheduleDao.get(bean.getCourseScheduleId());

        CourseHomework courseHomework = new CourseHomework();
        courseHomework.setId(bean.getCourseHomeworkId());
        courseHomework.setGroupType(courseSchedule.getGroupType());
        courseHomework.setCompletedNum(studentCourseHomeworkDao.countCompletedStudentNum(bean.getCourseHomeworkId()));

        CourseHomework temp = courseHomeworkService.get(courseHomework.getId());
        if (temp.getExpectNum().equals(courseHomework.getCompletedNum())) {
            CourseScheduleReview review = courseScheduleReviewDao.findByCourseScheduleAndStudent(bean.getCourseScheduleId(), bean.getUserId());
            if (Objects.nonNull(review)) {
                review.setHandHomework(YesOrNoEnum.YES.getCode());
                courseScheduleReviewDao.update(review);
            }
        }

        if(push){
            Map<Integer, String> userMap = new HashMap<>();
            userMap.put(courseSchedule.getActualTeacherId(), courseSchedule.getActualTeacherId().toString());
            SysUser user = teacherDao.getUser(bean.getUserId());
            String notifyUrl = "9?courseScheduleID=" + bean.getId() + "&userId=" + bean.getUserId() + "&studentCourseHomeworkId=" + bean.getId() + "&extra=0";
            String courseDate = DateUtil.dateToString(courseSchedule.getClassDate(), "yyyy年MM月dd日");
            sysMessageService.batchSendImMessage(MessageTypeEnum.IM_HOMEWORK_SUBMIT_PUSH,bean.getUserId().toString(),null,
                    new String[]{courseSchedule.getActualTeacherId().toString()},
                    null,courseDate,courseSchedule.getName());
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.HOMEWORK_SUBMIT_PUSH,
                    userMap, null, 0, notifyUrl, "TEACHER", courseSchedule.getName(), courseDate, user.getUsername());
            courseHomeworkService.update(courseHomework);
        }
    }

    @Override
    public CourseHomeworkStudentDetailDto findCourseHomeworkStudentDetail(Long courseScheduleID, Long userId) {
        if (Objects.isNull(courseScheduleID)) {
            throw new BizException("作业不存在");
        }
        CourseHomeworkStudentDetailDto courseHomeworkStudentDetail = studentCourseHomeworkDao.findCourseHomeworkStudentDetail(courseScheduleID, userId);
        if(Objects.nonNull(courseHomeworkStudentDetail)){
            SysUser student = teacherDao.getUser(userId.intValue());
            courseHomeworkStudentDetail.setStudentId(userId.intValue());
            courseHomeworkStudentDetail.setStudentName(student.getUsername());
            CourseSchedule courseSchedule = courseScheduleDao.get(courseHomeworkStudentDetail.getCourseScheduleId());
            if(Objects.nonNull(courseSchedule)){
                SysUser teacher = teacherDao.getUser(courseSchedule.getActualTeacherId());
                courseHomeworkStudentDetail.setTeacherId(courseSchedule.getActualTeacherId());
                courseHomeworkStudentDetail.setTeacherName(teacher.getRealName());
            }
        }
        return courseHomeworkStudentDetail;
    }

    @Override
    public boolean updateReplyStatus(Integer teacherId, Long courseHomeworkId, Integer studentId) {
        StudentCourseHomework byStudentAndCourseHomewok = studentCourseHomeworkDao.findByStudentAndCourseHomewok(courseHomeworkId, studentId);
        if (Objects.isNull(byStudentAndCourseHomewok)) {
            throw new BizException("作业记录不存在");
        }

        int hours = -1;
        if(Objects.nonNull(byStudentAndCourseHomewok.getSubmitTime())){
            hours = DateUtil.hoursBetween(byStudentAndCourseHomewok.getSubmitTime(), new Date());
        }
        if (byStudentAndCourseHomewok.getIsReplied() == YesOrNoEnum.YES && Integer.valueOf(1).equals(byStudentAndCourseHomewok.getIsRepliedTimely())) {
            return true;
        }
        if(hours>=0&&hours<=12){
            byStudentAndCourseHomewok.setIsRepliedTimely(1);
        }
        CourseHomework byTeacherAndCourseHomewok = courseHomeworkDao.findByTeacherAndCourseHomewok(courseHomeworkId, teacherId);
        if (Objects.nonNull(byTeacherAndCourseHomewok)) {
            byStudentAndCourseHomewok.setIsReplied(YesOrNoEnum.YES);
            studentCourseHomeworkDao.update(byStudentAndCourseHomewok);
        }
        return true;
    }

    @Override
    public PageInfo<StudentCourseHomework> queryPage(QueryInfo queryInfo) {
        SysUser user = sysUserFeignService.queryUserInfo();
        CourseHomeworkQueryInfo courseHomeworkQueryInfo = (CourseHomeworkQueryInfo) queryInfo;
        courseHomeworkQueryInfo.setUserId(Long.valueOf(user.getId()));
        return super.queryPage(queryInfo);
    }

    @Override
    public List<StudentCourseHomework> findStudentCourseHomeworkByCourse(Long courseSchedule, String userName) {
        if (Objects.isNull(courseSchedule)) {
            throw new BizException("请指定课程");
        }
        return studentCourseHomeworkDao.findByCourseSchedule(courseSchedule, userName);
    }

    @Override
    public void homeworkRemind() {
        List<StudentCourseHomework> list = studentCourseHomeworkDao.queryNotDoneHomeworkList();
        StringBuffer sb;
        for (StudentCourseHomework sch : list) {
            sb = new StringBuffer("courseScheduleID=").append(sch.getCourseScheduleId()).append("&studentCourseHomeworkId=").append(sch.getId());
            Map<Integer, String> receivers = new HashMap<Integer, String>(1);
            receivers.put(sch.getUserId(), sch.getUserId() + "");
            sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_JOB_MESSAGE, receivers, null, 0, "3?" + sb, "STUDENT", sch.getCourseHomework()
                    .getMusicGroup().getName(), sch.getCourseHomework().getClassGroup().getName());
        }
    }

    @Override
    public PageInfo<StudentCourseHomeworkDto> findStudentCourseHomeworks(QueryInfo queryInfo) {
        PageInfo<StudentCourseHomeworkDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentCourseHomeworkDto> dataList = null;
        int count = studentCourseHomeworkDao.countStudentCourseHomeworks(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentCourseHomeworkDao.findStudentCourseHomeworks(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfo findStudentHomeworkRecord(CourseHomeworkQueryInfo queryInfo) {
        PageInfo<StudentHomeworkRecordDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentHomeworkRecordDto> dataList = null;
        int count = studentCourseHomeworkDao.countStudentHomeworkRecord(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentCourseHomeworkDao.findStudentHomeworkRecord(params);
            List<Integer> teacherIds = dataList.stream().map(StudentHomeworkRecordDto::getTeacherId).collect(Collectors.toList());
            List<SimpleUserDto> teachers = teacherDao.findSimpleTeachers(teacherIds);
            Map<Integer, String> idNameMap = teachers.stream().collect(Collectors.toMap(SimpleUserDto::getUserId, u -> u.getUserName()));
            for (StudentHomeworkRecordDto studentHomeworkRecordDto : dataList) {
                studentHomeworkRecordDto.setTeacherName(idNameMap.get(studentHomeworkRecordDto.getTeacherId()));
            }
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }
}
