package com.ym.mec.biz.service.impl;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import javax.annotation.Resource;

import com.ym.mec.biz.dal.entity.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.entity.SysUserRole;
import com.ym.mec.biz.dal.dao.MusicGroupDao;
import com.ym.mec.biz.dal.dao.MusicGroupStudentFeeDao;
import com.ym.mec.biz.dal.dao.MusicGroupSubjectPlanDao;
import com.ym.mec.biz.dal.dao.StudentRegistrationDao;
import com.ym.mec.biz.dal.dao.SubjectDao;
import com.ym.mec.biz.dal.dao.SysConfigDao;
import com.ym.mec.biz.dal.dao.SysUserCashAccountDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dto.StudentAddDto;
import com.ym.mec.biz.dal.dto.StudentApplyDetailDto;
import com.ym.mec.biz.dal.dto.StudentFeeDetailDto;
import com.ym.mec.biz.dal.dto.StudentInfo;
import com.ym.mec.biz.dal.enums.ClassGroupStudentStatusEnum;
import com.ym.mec.biz.dal.enums.ClassGroupTypeEnum;
import com.ym.mec.biz.dal.enums.DealStatusEnum;
import com.ym.mec.biz.dal.enums.GoodsType;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.KitGroupPurchaseTypeEnum;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.OrderDetailTypeEnum;
import com.ym.mec.biz.dal.enums.OrderTypeEnum;
import com.ym.mec.biz.dal.enums.PaymentStatusEnum;
import com.ym.mec.biz.dal.enums.PlatformCashAccountDetailTypeEnum;
import com.ym.mec.biz.dal.page.StudentRegistrationQueryInfo;
import com.ym.mec.biz.service.ClassGroupService;
import com.ym.mec.biz.service.ClassGroupStudentMapperService;
import com.ym.mec.biz.service.ContractService;
import com.ym.mec.biz.service.CourseScheduleService;
import com.ym.mec.biz.service.CourseScheduleStudentPaymentService;
import com.ym.mec.biz.service.MusicGroupPaymentCalenderService;
import com.ym.mec.biz.service.MusicGroupSubjectPlanService;
import com.ym.mec.biz.service.StudentPaymentOrderDetailService;
import com.ym.mec.biz.service.StudentPaymentOrderService;
import com.ym.mec.biz.service.StudentRegistrationService;
import com.ym.mec.biz.service.SubjectService;
import com.ym.mec.biz.service.SysConfigService;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.biz.service.SysUserCashAccountDetailService;
import com.ym.mec.biz.service.SysUserCashAccountService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImGroupMember;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.entity.ImResult;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.redis.service.RedisCache;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.http.HttpUtil;

@Service
public class StudentRegistrationServiceImpl extends BaseServiceImpl<Long, StudentRegistration> implements StudentRegistrationService {

    private static final Logger LOGGER = LoggerFactory.getLogger(StudentRegistrationServiceImpl.class);

    @Resource
    private StudentRegistrationDao studentRegistrationDao;
    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;
    @Autowired
    private StudentPaymentOrderDetailService studentPaymentOrderDetailService;
    @Autowired
    private CourseScheduleStudentPaymentService courseScheduleStudentPaymentService;
    @Autowired
    private CourseScheduleService courseScheduleService;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private ClassGroupService classGroupDao;
    @Autowired
    private MusicGroupStudentFeeDao musicGroupStudentFeeDao;
    @Autowired
    private ClassGroupStudentMapperService classGroupStudentMapperDao;
    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    private MusicGroupSubjectPlanDao musicGroupSubjectPlanDao;
    @Autowired
    private SubjectDao subjectDao;
    @Autowired
    private SysUserCashAccountDao sysUserCashAccountDao;
    @Autowired
    private ContractService contractService;
    @Autowired
    private ImFeignService imFeignService;
    @Autowired
    private MusicGroupSubjectPlanService musicGroupSubjectPlanService;
    @Autowired
    private MusicGroupPaymentCalenderService musicGroupPaymentCalenderService;
    @Autowired
    private SubjectService subjectService;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private SysUserCashAccountDetailService sysUserCashAccountDetailService;
    @Autowired
    private SysUserCashAccountService sysUserCashAccountService;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private RedisCache<String, Object> redisCache;

    @Override
    public BaseDAO<Long, StudentRegistration> getDAO() {
        return studentRegistrationDao;
    }

    @Override
    public PageInfo<StudentApplyDetailDto> queryStudentDetailPage(StudentRegistrationQueryInfo queryInfo) {
        PageInfo<StudentApplyDetailDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentApplyDetailDto> dataList = null;
        int count = studentRegistrationDao.queryStudentDetailCount(params);
        if(queryInfo.getIsExport() && count > 50000){
            throw new BizException("数据集太大，不能导出.最大数据集不能超过50000");
        }
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentRegistrationDao.queryStudentDetailPage(params);
            Set<Integer> subjectIds = dataList.stream().map(e -> e.getActualSubjectId()).collect(Collectors.toSet());
            Map<Integer, String> subjectMap = MapUtil.convertIntegerMap(musicGroupSubjectPlanDao.findSubjectMap(subjectIds, queryInfo.getMusicGroupId()));
            dataList.forEach(e -> {
                if (StringUtils.isNotBlank(subjectMap.get(e.getActualSubjectId()))) {
                    e.setKitGroupPurchaseTypeEnum(KitGroupPurchaseTypeEnum.valueOf(subjectMap.get(e.getActualSubjectId())));
                }
            });
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchUpdateSubject(Integer userId, Integer subId, String musicGroupId) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            throw new BizException("获取用户信息失败");
        }
        //获取当前学员的当前声部
        StudentRegistration studentRegistration = studentRegistrationDao.queryByUserIdAndMusicGroupId(userId, musicGroupId);
        if (studentRegistration == null) {
            throw new BizException("学员信息不存在");
        }
        //批量调剂（未缴费学员）
        int i = studentRegistrationDao.batchUpdateSubject(userId, subId, musicGroupId);
        if (i > 0) {
            //当前专业报名人数减一
            musicGroupSubjectPlanService.addApplyStudentNum(musicGroupId, studentRegistration.getActualSubjectId(), -1);
            //修改专业已报名人数
            musicGroupSubjectPlanService.addApplyStudentNum(musicGroupId, subId, 1);
            MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
            Set<Integer> roleIds = new HashSet<>(2);
            roleIds.add(SysUserRole.SECTION_MANAGER);
            roleIds.add(SysUserRole.OPERATION_EXECUTIVE);

            sysMessageService.batchSeoMessage(musicGroupDao.queryUserIdByRoleId(roleIds, musicGroup.getOrganId()), MessageTypeEnum.BACKSTAGE_STUDENT_SUBJECT_CHANGE, "", musicGroup.getName());
        }
        return i;
    }

    @Override
    public StudentFeeDetailDto queryFeeDetail(Integer studentId, String musicGroupId) {
        return studentRegistrationDao.queryFeeDetail(studentId, musicGroupId);
    }

    @Override
    public List<Map<String, Object>> getNoClassStuBySubjectId(String musicGroupId, String actualSubjectId) {
        List<StudentRegistration> students = studentRegistrationDao.getNoClassStuBySubjectId(musicGroupId, actualSubjectId);
        List<Map<String, Object>> mapArrayList = new ArrayList<>();
        if (students != null && students.size() > 0) {
            String[] subjectIds = actualSubjectId.split(",");
            if (subjectIds != null && subjectIds.length > 0) {
                List<Map<Integer, String>> subjectNames = subjectDao.queryNameByIds(actualSubjectId);
                Map<Integer, String> subjectNameMap = MapUtil.convertMybatisMap(subjectNames);
                for (int i = 0; i < subjectIds.length; i++) {
                    Map<String, Object> resultMap = new HashMap<>(3);
                    int subjectId = Integer.parseInt(subjectIds[i]);
                    List<StudentRegistration> collect = students.stream().filter(e -> e.getActualSubjectId().equals(subjectId)).collect(Collectors.toList());
                    resultMap.put("subjectId", subjectId);
                    resultMap.put("subjectName", subjectNameMap.get(subjectId));
                    resultMap.put("rows", collect);
                    mapArrayList.add(resultMap);
                }
            }
        }
        return mapArrayList;
    }

    @Override
    public List<MusicGroupSubjectPlan> getNoClassStuCountByMusicGroupId(String musicGroupId) {
        return studentRegistrationDao.getNoClassStuCountByMusicGroupId(musicGroupId);
    }

    @Override
    public Integer updateByUserIdAndMusicGroupId(StudentRegistration studentRegistration) {
        return studentRegistrationDao.updateByUserIdAndMusicGroupId(studentRegistration);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public StudentRegistration addStudent(StudentRegistration studentRegistration) throws Exception {
        String key = "_student_registration";
        long threadId = Thread.currentThread().getId();
        if (!redisCache.getLocked(key, threadId, 10)) {
            throw new BizException("系统繁忙，请稍后再试");
        }
        try {
            Date date = new Date();
            Integer userId = 0;
            SysUser sysUser = studentRegistrationDao.getSysUserByPhone(studentRegistration.getParentsPhone());
            if (sysUser == null) {
                sysUser = new SysUser();
                sysUser.setPhone(studentRegistration.getParentsPhone());
                sysUser.setCreateTime(date);
                sysUser.setUpdateTime(date);
                sysUser.setLockFlag(0);
                sysUser.setDelFlag(com.ym.mec.auth.api.enums.YesOrNoEnum.NO);
                sysUser.setUserType("STUDENT");
                sysUser.setGender(studentRegistration.getGender());
                sysUser.setOrganId(studentRegistration.getOrganId());
                sysUser.setRealName(studentRegistration.getParentsName());
                sysUser.setUsername(studentRegistration.getName());
                sysUser.setIdCardNo(studentRegistration.getIdCardNo());
                sysUser.setBirthdate(studentRegistration.getBirthdate());
                teacherDao.addSysUser(sysUser);
                userId = sysUser.getId();
                // 添加用户现金账户
                sysUserCashAccountDao.insert(new SysUserCashAccount(userId, "CNY"));
                // 注册到融云
                if (StringUtils.isEmpty(sysUser.getAvatar())) {
                    sysUser.setAvatar(sysConfigDao.findConfigValue(SysConfigService.USER_DEFAULT_HEAD_URL));
                }
                ImResult register = imFeignService.register(new ImUserModel(userId.toString(), sysUser.getUsername(), sysUser.getAvatar()));
                sysUser.setImToken(register.getToken());
            } else {
                if (!sysUser.getUserType().contains("STUDENT")) {
                    sysUser.setUserType(sysUser.getUserType() + ",STUDENT");
                }
                sysUser.setPhone(studentRegistration.getParentsPhone());
                sysUser.setOrganId(studentRegistration.getOrganId());
                sysUser.setRealName(studentRegistration.getParentsName());
                sysUser.setUsername(studentRegistration.getName());
                sysUser.setGender(studentRegistration.getGender());
                sysUser.setBirthdate(studentRegistration.getBirthdate());
                sysUser.setCreateTime(date);
                sysUser.setUpdateTime(date);
                teacherDao.updateUser(sysUser);
            }

            studentRegistration.setActualSubjectId(studentRegistration.getSubjectId());
            studentRegistration.setCreateTime(date);
            studentRegistration.setUpdateTime(date);
            studentRegistration.setUserId(sysUser.getId());
            studentRegistrationDao.insert(studentRegistration);
            studentRegistrationDao.updateCurrentClass(studentRegistration);
            // 增加报名学生数
            musicGroupSubjectPlanService.addApplyStudentNum(studentRegistration.getMusicGroupId(), studentRegistration.getSubjectId(), 1);
            // 报名成功后，发送短信
            // String studentApplyUrl = sysConfigDao.findConfigValue(SysConfigService.STUDENT_APPLY_URL) + studentRegistration.getMusicGroupId();
            String serverPhone = sysConfigDao.findConfigValue(SysConfigService.SERVER_PHONE);
            Subject subject = subjectDao.get(studentRegistration.getActualSubjectId());
            // MusicGroup musicGroup = musicGroupDao.get(studentRegistration.getMusicGroupId());

            Map<Integer, String> map = new HashMap<>(1);
            map.put(studentRegistration.getUserId(), studentRegistration.getParentsPhone());
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI, MessageTypeEnum.SMS_APPLY_MESSAGE, map, null, 0, "", "",
                    studentRegistration.getParentsName(), subject.getName(), serverPhone);
        } finally {
            redisCache.releaseLocked(key, threadId);
        }
        return studentRegistration;
    }

    @Override
    public StudentInfo queryStudentInfo(Integer userId) {
        return studentRegistrationDao.queryStudentInfo(userId);
    }

    @Override
    @Transactional
    public StudentPaymentOrder addOrder(StudentRegistration studentRegistration, BigDecimal amount, String orderNo, String paymentChannel, BigDecimal courseFee, List<MusicGroupSubjectGoodsGroup> goodsGroups, List<Goods> goodsList, List<Goods> otherGoodsList) throws Exception {
        Date date = new Date();
        StudentPaymentOrder studentPaymentOrder = new StudentPaymentOrder();
        studentPaymentOrder.setUserId(studentRegistration.getUserId());
        studentPaymentOrder.setGroupType(GroupType.MUSIC);
        studentPaymentOrder.setOrderNo(orderNo);
        studentPaymentOrder.setType(OrderTypeEnum.APPLY);
        studentPaymentOrder.setExpectAmount(amount);
        studentPaymentOrder.setActualAmount(amount);
        studentPaymentOrder.setStatus(DealStatusEnum.ING);
        studentPaymentOrder.setPaymentChannel(paymentChannel);
        studentPaymentOrder.setMusicGroupId(studentRegistration.getMusicGroupId());
        studentPaymentOrderService.insert(studentPaymentOrder);

        ArrayList<StudentPaymentOrderDetail> studentPaymentOrderDetailList = new ArrayList<>();
        StudentPaymentOrderDetail studentPaymentOrderDetail = new StudentPaymentOrderDetail();
        studentPaymentOrderDetail.setType(OrderDetailTypeEnum.COURSE);
        studentPaymentOrderDetail.setPrice(courseFee);
        studentPaymentOrderDetail.setCreateTime(date);
        studentPaymentOrderDetail.setUpdateTime(date);
        studentPaymentOrderDetail.setPaymentOrderId(studentPaymentOrder.getId());
        studentPaymentOrderDetailList.add(studentPaymentOrderDetail);
        //乐器及打包辅件
        if (goodsGroups != null && goodsGroups.size() > 0) {
            for (MusicGroupSubjectGoodsGroup goodsGroup : goodsGroups) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4goodsGroup = new StudentPaymentOrderDetail();
                OrderDetailTypeEnum type = null;
                if (goodsGroup.getType().equals(GoodsType.INSTRUMENT)) {
                    type = OrderDetailTypeEnum.MUSICAL;
                } else if (goodsGroup.getType().equals(GoodsType.ACCESSORIES)) {
                    type = OrderDetailTypeEnum.ACCESSORIES;
                } else if (goodsGroup.getType().equals(GoodsType.OTHER)) {
                    type = OrderDetailTypeEnum.TEACHING;
                }
                studentPaymentOrderDetail4goodsGroup.setType(type);
                studentPaymentOrderDetail4goodsGroup.setPrice(goodsGroup.getPrice());
                studentPaymentOrderDetail4goodsGroup.setGoodsIdList(goodsGroup.getGoodsIdList());
                studentPaymentOrderDetail4goodsGroup.setCreateTime(date);
                studentPaymentOrderDetail4goodsGroup.setUpdateTime(date);
                studentPaymentOrderDetail4goodsGroup.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetail4goodsGroup.setKitGroupPurchaseType(goodsGroup.getKitGroupPurchaseType());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4goodsGroup);
            }
        }

        //单独辅件
        if (goodsList != null && goodsList.size() > 0) {
            for (Goods goods : goodsList) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4goods = new StudentPaymentOrderDetail();
                OrderDetailTypeEnum orderDetailType = goods.getType().equals(GoodsType.INSTRUMENT) ? OrderDetailTypeEnum.MUSICAL : OrderDetailTypeEnum.ACCESSORIES;
                studentPaymentOrderDetail4goods.setType(orderDetailType);
                studentPaymentOrderDetail4goods.setPrice(goods.getGroupPurchasePrice());
                studentPaymentOrderDetail4goods.setGoodsIdList(goods.getId().toString());
                studentPaymentOrderDetail4goods.setCreateTime(date);
                studentPaymentOrderDetail4goods.setUpdateTime(date);
                studentPaymentOrderDetail4goods.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4goods);
            }
        }
        //单独教谱
        if (otherGoodsList != null && otherGoodsList.size() > 0) {
            for (Goods goods : otherGoodsList) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4otherGoods = new StudentPaymentOrderDetail();
                studentPaymentOrderDetail4otherGoods.setType(OrderDetailTypeEnum.TEACHING);
                studentPaymentOrderDetail4otherGoods.setPrice(goods.getGroupPurchasePrice());
                studentPaymentOrderDetail4otherGoods.setGoodsIdList(goods.getId().toString());
                studentPaymentOrderDetail4otherGoods.setCreateTime(date);
                studentPaymentOrderDetail4otherGoods.setUpdateTime(date);
                studentPaymentOrderDetail4otherGoods.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4otherGoods);
            }
        }

        studentPaymentOrderDetailService.batchAdd(studentPaymentOrderDetailList);

        //增加缴费学生数
        boolean updateFlag = false;
        for (int i = 0; i < 10; i++) {
            if (amount.compareTo(BigDecimal.ZERO) == 0) { //0元订单不增加报名人数
                updateFlag = true;
                break;
            }
            MusicGroupSubjectPlan musicOneSubjectClassPlan = musicGroupSubjectPlanService.getMusicOneSubjectClassPlan(studentRegistration.getMusicGroupId(), studentRegistration.getActualSubjectId());
            int paidNum = musicOneSubjectClassPlan.getPaidStudentNum() == null ? 0 : musicOneSubjectClassPlan.getPaidStudentNum();
            musicOneSubjectClassPlan.setPaidStudentNum(paidNum + 1);
            if (studentRegistration.getTemporaryCourseFee() == null && musicOneSubjectClassPlan.getPaidStudentNum() > musicOneSubjectClassPlan.getExpectedStudentNum()) {
                throw new BizException("乐团人数已满");
            }
            int count = musicGroupSubjectPlanService.update(musicOneSubjectClassPlan);
            if (count >= 1) {
                updateFlag = true;
                break;
            }
        }
        if (!updateFlag) {
            throw new BizException("系统繁忙，更新缴费人数失败");
        }

        return studentPaymentOrder;
    }

    @Override
    @Transactional
    public StudentPaymentOrder reAddOrder(Integer userId, BigDecimal amount, String orderNo, String paymentChannel, BigDecimal courseFee, List<MusicGroupSubjectGoodsGroup> goodsGroups, List<Goods> goodsList, List<Goods> otherGoodsList, String musicGroupId, StudentPaymentOrder oldOrder) {
        //关闭老订单
        oldOrder.setStatus(DealStatusEnum.CLOSE);
        studentPaymentOrderService.update(oldOrder);
        if (oldOrder.getBalancePaymentAmount() != null && oldOrder.getBalancePaymentAmount().compareTo(BigDecimal.ZERO) > 0) {
            sysUserCashAccountService.updateBalance(oldOrder.getUserId(), oldOrder.getBalancePaymentAmount(), PlatformCashAccountDetailTypeEnum.REFUNDS, "关闭订单");
        }
        Date date = new Date();
        StudentPaymentOrder studentPaymentOrder = new StudentPaymentOrder();
        studentPaymentOrder.setUserId(userId);
        studentPaymentOrder.setGroupType(GroupType.MUSIC);
        studentPaymentOrder.setOrderNo(orderNo);
        studentPaymentOrder.setType(OrderTypeEnum.APPLY);
        studentPaymentOrder.setExpectAmount(amount);
        studentPaymentOrder.setActualAmount(amount);
        studentPaymentOrder.setStatus(DealStatusEnum.ING);
        studentPaymentOrder.setPaymentChannel(paymentChannel);
        studentPaymentOrder.setMusicGroupId(musicGroupId);
        studentPaymentOrderService.insert(studentPaymentOrder);

        ArrayList<StudentPaymentOrderDetail> studentPaymentOrderDetailList = new ArrayList<>();
        StudentPaymentOrderDetail studentPaymentOrderDetail = new StudentPaymentOrderDetail();
        studentPaymentOrderDetail.setType(OrderDetailTypeEnum.COURSE);
        studentPaymentOrderDetail.setPrice(courseFee);
        studentPaymentOrderDetail.setCreateTime(date);
        studentPaymentOrderDetail.setUpdateTime(date);
        studentPaymentOrderDetail.setPaymentOrderId(studentPaymentOrder.getId());
        studentPaymentOrderDetailList.add(studentPaymentOrderDetail);
        //乐器及打包辅件
        if (goodsGroups != null && goodsGroups.size() > 0) {
            for (MusicGroupSubjectGoodsGroup goodsGroup : goodsGroups) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4goodsGroup = new StudentPaymentOrderDetail();
                OrderDetailTypeEnum type = null;
                if (goodsGroup.getType().equals(GoodsType.INSTRUMENT)) {
                    type = OrderDetailTypeEnum.MUSICAL;
                } else if (goodsGroup.getType().equals(GoodsType.ACCESSORIES)) {
                    type = OrderDetailTypeEnum.ACCESSORIES;
                } else if (goodsGroup.getType().equals(GoodsType.OTHER)) {
                    type = OrderDetailTypeEnum.TEACHING;
                }
                studentPaymentOrderDetail4goodsGroup.setType(type);
                studentPaymentOrderDetail4goodsGroup.setPrice(goodsGroup.getPrice());
                studentPaymentOrderDetail4goodsGroup.setGoodsIdList(goodsGroup.getGoodsIdList());
                studentPaymentOrderDetail4goodsGroup.setCreateTime(date);
                studentPaymentOrderDetail4goodsGroup.setUpdateTime(date);
                studentPaymentOrderDetail4goodsGroup.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetail4goodsGroup.setKitGroupPurchaseType(goodsGroup.getKitGroupPurchaseType());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4goodsGroup);
            }
        }

        //单独辅件
        if (goodsList != null && goodsList.size() > 0) {
            for (Goods goods : goodsList) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4goods = new StudentPaymentOrderDetail();
                OrderDetailTypeEnum orderDetailType = goods.getType().equals(GoodsType.INSTRUMENT) ? OrderDetailTypeEnum.MUSICAL : OrderDetailTypeEnum.ACCESSORIES;
                studentPaymentOrderDetail4goods.setType(orderDetailType);
                studentPaymentOrderDetail4goods.setPrice(goods.getGroupPurchasePrice());
                studentPaymentOrderDetail4goods.setGoodsIdList(goods.getId().toString());
                studentPaymentOrderDetail4goods.setCreateTime(date);
                studentPaymentOrderDetail4goods.setUpdateTime(date);
                studentPaymentOrderDetail4goods.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4goods);

            }
        }
        //单独教谱
        if (otherGoodsList != null && otherGoodsList.size() > 0) {
            for (Goods goods : otherGoodsList) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4otherGoods = new StudentPaymentOrderDetail();
                studentPaymentOrderDetail4otherGoods.setType(OrderDetailTypeEnum.TEACHING);
                studentPaymentOrderDetail4otherGoods.setPrice(goods.getGroupPurchasePrice());
                studentPaymentOrderDetail4otherGoods.setGoodsIdList(goods.getId().toString());
                studentPaymentOrderDetail4otherGoods.setCreateTime(date);
                studentPaymentOrderDetail4otherGoods.setUpdateTime(date);
                studentPaymentOrderDetail4otherGoods.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4otherGoods);
            }
        }

        studentPaymentOrderDetailService.batchAdd(studentPaymentOrderDetailList);

        return studentPaymentOrder;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public StudentRegistration queryByUserIdAndMusicGroupId(Integer userId, String musicGroupId) {
        StudentRegistration registration = studentRegistrationDao.queryByUserIdAndMusicGroupId(userId, musicGroupId);
        if (registration == null) {
            return null;
        }
        SysUserCashAccount account = sysUserCashAccountDao.getLocked(userId);
        registration.setBalance(account.getBalance());
        registration.setTransferStudent(registration.getTemporaryCourseFee() == null ? 0 : 1);
        return registration;
    }

    @Override
    public StudentRegistration getByPhoneAndMusicGroupId(String musicGroupId, String parentsPhone) {
        return studentRegistrationDao.getByPhoneAndMusicGroupId(musicGroupId, parentsPhone);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertStudent(StudentAddDto studentAddDto) throws Exception {
        StudentRegistration studentRegistration = studentAddDto.getStudentRegistration();
        if (studentRegistration == null) {
            throw new BizException("参数校验失败");
        }
        /*if(studentAddDto.getSignClassId() == null && studentAddDto.getMixClassId() == null){
            throw new BizException("参数校验失败: 单技班或合奏班必填一项");
        }*/
        String musicGroupId = studentRegistration.getMusicGroupId();

        StudentRegistration phoneAndMusicGroupId = studentRegistrationDao.getByPhoneAndMusicGroupId(musicGroupId, studentRegistration.getParentsPhone());
        if (phoneAndMusicGroupId != null && phoneAndMusicGroupId.getMusicGroupStatus() != ClassGroupStudentStatusEnum.QUIT) {
            throw new BizException("该学员已存在");
        } else {
            SysUser sysUser = studentRegistrationDao.getSysUserByPhone(studentRegistration.getParentsPhone());
            MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
            Integer userId;
            if (sysUser == null) {
                //新增user
                sysUser = new SysUser();
                sysUser.setPhone(studentRegistration.getParentsPhone());
                sysUser.setRealName(studentRegistration.getParentsName());
                sysUser.setUsername(studentRegistration.getName());
                sysUser.setGender(studentRegistration.getGender());
                sysUser.setUserType("STUDENT");
                sysUser.setIdCardNo(studentRegistration.getIdCardNo());
                sysUser.setOrganId(musicGroup.getOrganId());
                sysUser.setBirthdate(studentRegistration.getBirthdate());
                teacherDao.addSysUser(sysUser);
                userId = sysUser.getId();
                //添加用户现金账户
                sysUserCashAccountDao.insert(new SysUserCashAccount(userId, "CNY"));
                //添加用户电子签章账户
//                contractService.register(userId, sysUser.getRealName(), sysUser.getIdCardNo(), sysUser.getPhone());
                ImResult register = imFeignService.register(new ImUserModel(userId.toString(), sysUser.getUsername(), null));
                sysUser.setImToken(register.getToken());
                teacherDao.updateUser(sysUser);
            } else {
                if (!sysUser.getUserType().contains("STUDENT")) {
                    sysUser.setUserType(sysUser.getUserType() + ",STUDENT");
                }
                sysUser.setPhone(studentRegistration.getParentsPhone());
                sysUser.setRealName(studentRegistration.getParentsName());
                sysUser.setUsername(studentRegistration.getName());
                sysUser.setGender(studentRegistration.getGender());
                sysUser.setOrganId(musicGroup.getOrganId());
                sysUser.setBirthdate(studentRegistration.getBirthdate());
                teacherDao.updateUser(sysUser);
                userId = sysUser.getId();
            }
            studentRegistration.setActualSubjectId(studentRegistration.getSubjectId());
            studentRegistration.setUserId(userId);
            studentRegistration.setPaymentStatus(PaymentStatusEnum.OPEN);
            studentRegistration.setMusicGroupStatus(ClassGroupStudentStatusEnum.NORMAL);
            studentRegistration.setTemporaryCourseFee(studentAddDto.getTemporaryCourseFee());
            //学生报名表
            Set<Integer> roleIds = new HashSet<>(1);
            roleIds.add(SysUserRole.OPERATION_EXECUTIVE);

            sysMessageService.batchSeoMessage(musicGroupDao.queryUserIdByRoleId(roleIds, musicGroup.getOrganId()), MessageTypeEnum.BACKSTAGE_MUSIC_GROUP_ADD_STUDENT, "", musicGroup.getName(), studentRegistration.getName());
            //增加报名学生数
            musicGroupSubjectPlanService.addApplyStudentNum(musicGroupId, studentRegistration.getSubjectId(), 1);
            //添加班级关联
            addStudents(studentAddDto, userId);
            Date date = new Date();
            //保存乐团学生费用表
            if (studentAddDto.getCourseFee() == null) {
                MusicGroupSubjectPlan musicOneSubjectClassPlan = musicGroupSubjectPlanDao.getMusicOneSubjectClassPlan(musicGroupId, studentRegistration.getActualSubjectId());
                studentAddDto.setCourseFee(musicOneSubjectClassPlan.getFee());
            }
            Date nextPaymentDate = musicGroupPaymentCalenderService.getNextPaymentDate(musicGroupId, null, null);

            StudentPaymentOrder waitPayOrder = studentPaymentOrderService.findMusicGroupApplyOrderByStatus(userId, studentRegistration.getMusicGroupId(), DealStatusEnum.WAIT_PAY);

            if (waitPayOrder != null) {
                waitPayOrder.setStatus(DealStatusEnum.CLOSE);
                studentPaymentOrderService.update(waitPayOrder);
            }
            //生成订单
            StudentPaymentOrder studentPaymentOrder = new StudentPaymentOrder();
            studentPaymentOrder.setUserId(userId);
            studentPaymentOrder.setGroupType(GroupType.MUSIC);
            studentPaymentOrder.setType(OrderTypeEnum.APPLY);
            studentPaymentOrder.setStatus(DealStatusEnum.WAIT_PAY);
            studentPaymentOrder.setMusicGroupId(musicGroupId);
            studentPaymentOrder.setClassGroupId(studentAddDto.getSignClassId());
            studentPaymentOrder.setOrganId(musicGroup.getOrganId());
            studentPaymentOrder.setRoutingOrganId(42);

            //生成订单明细
            List<StudentPaymentOrderDetail> studentPaymentOrderDetails = studentAddDto.getStudentPaymentOrderDetails();
            if (studentPaymentOrderDetails == null) {
                studentPaymentOrderDetails = new ArrayList<>();
            } else {
                studentPaymentOrderDetails.forEach(e -> {
                    if (e.getPrice() == null) {
                        throw new BizException("请补全订单信息");
                    }
                });
            }
            StudentPaymentOrderDetail studentPaymentOrderDetail = new StudentPaymentOrderDetail();
            studentPaymentOrderDetail.setPrice(studentAddDto.getTemporaryCourseFee());
            studentPaymentOrderDetail.setType(OrderDetailTypeEnum.COURSE);
            studentPaymentOrderDetails.add(studentPaymentOrderDetail);
            //汇总金额

            BigDecimal reduce = studentPaymentOrderDetails.stream().map(e -> e.getPrice()).reduce(BigDecimal.ZERO, BigDecimal::add);
            MusicGroupStudentFee.PaymentStatus paymentStatus = MusicGroupStudentFee.PaymentStatus.NON_PAYMENT;
            if (reduce == BigDecimal.ZERO) {
                paymentStatus = MusicGroupStudentFee.PaymentStatus.PAID_COMPLETED;
                studentPaymentOrder.setStatus(DealStatusEnum.SUCCESS);
                studentRegistration.setPaymentStatus(PaymentStatusEnum.YES);
                studentPaymentOrder.setActualAmount(reduce);
                studentPaymentOrder.setPayTime(date);
            }
            musicGroupStudentFeeDao.insert(new MusicGroupStudentFee(musicGroupId,
                    userId, studentRegistration.getSubjectId(), studentAddDto.getCourseFee(),
                    nextPaymentDate, studentAddDto.getTemporaryCourseFee(), paymentStatus));
            musicGroupStudentFeeDao.updateCalender(userId, musicGroupId);
            studentPaymentOrder.setExpectAmount(reduce);
            //当前学员是否以前存在过当前乐团
            if (phoneAndMusicGroupId != null) {
                studentRegistration.setId(phoneAndMusicGroupId.getId());
                studentRegistrationDao.update(studentRegistration);
            } else {
                studentRegistrationDao.insertBasic(studentRegistration);
            }
            studentRegistrationDao.updateCurrentClass(studentRegistration);
            studentPaymentOrderService.insert(studentPaymentOrder);
            studentPaymentOrderDetails.forEach(e -> {
                e.setPaymentOrderId(studentPaymentOrder.getId());
            });
            studentPaymentOrderDetailService.batchAdd(studentPaymentOrderDetails);
            if (musicGroup.getOwnershipType() != null && musicGroup.getOwnershipType() == CooperationOrgan.OwnershipType.OWN) {
                Map<Integer, String> receivers = new HashMap<>(1);
                receivers.put(studentRegistration.getUserId(), studentRegistration.getParentsPhone());
                if (reduce == BigDecimal.ZERO) {
                    sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI, MessageTypeEnum.SMS_MUSIC_GROUP_ADD_STUDENT_FREE,
                            receivers, null, 0, null, "", studentRegistration.getParentsName(), studentRegistration.getName(), musicGroup.getName());
                } else {
                    String studentApplyUrl = sysConfigDao.findConfigValue(SysConfigService.STUDENT_APPLY_URL) + musicGroupId;
                    sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI, MessageTypeEnum.SMS_MUSIC_GROUP_ADD_STUDENT, receivers, null, 0, null, "", studentRegistration.getParentsName(), studentRegistration.getName(), musicGroup.getName(), HttpUtil.getSortUrl(studentApplyUrl));
                }
            }
            return userId;
        }
    }

    private List<CourseSchedule> addStudent(Integer classGroupId, Integer userId, Date date, List<CourseSchedule> courseScheduleList) {
        ClassGroup classGroup = classGroupDao.get(classGroupId);
        ClassGroupStudentMapper classGroupStudentMapper = classGroupStudentMapperDao.findClassStudentMapperByUserIdAndClassGroupId(userId, classGroupId, null);
        if (classGroupStudentMapper != null) {
            classGroupStudentMapper.setStatus(ClassGroupStudentStatusEnum.NORMAL);
            classGroupStudentMapper.setClassGroupId(classGroup.getId());
            classGroupStudentMapperDao.update(classGroupStudentMapper);
        } else {
            classGroupStudentMapper = new ClassGroupStudentMapper();
            classGroupStudentMapper.setMusicGroupId(classGroup.getMusicGroupId());
            classGroupStudentMapper.setClassGroupId(classGroupId);
            classGroupStudentMapper.setUserId(userId);
            classGroupStudentMapper.setCreateTime(date);
            classGroupStudentMapper.setStatus(ClassGroupStudentStatusEnum.NORMAL);
            classGroupStudentMapper.setGroupType(GroupType.MUSIC);
            classGroupStudentMapperDao.insert(classGroupStudentMapper);
        }
        ImGroupMember[] imGroupMembers = {new ImGroupMember(userId.toString())};
        imFeignService.groupJoin(new ImGroupModel(classGroupId.toString(), imGroupMembers, classGroup.getName()));
        //2、班级人数调整
//        classGroupDao.updateClassStudentNum(classGroupId.longValue(), 1);
        //3、学生加入新班级未开始课程
        List<CourseSchedule> noStartCoursesByClassGroupId = courseScheduleService.findNoStartCoursesByClassGroupId(classGroupId);
        if (noStartCoursesByClassGroupId != null && noStartCoursesByClassGroupId.size() > 0) {
            courseScheduleList.addAll(noStartCoursesByClassGroupId);
        }
        return courseScheduleList;
    }

    public void addStudents(StudentAddDto studentAddDto, Integer userId) {
        Date nowDate = new Date();

        String musicGroupId = studentAddDto.getStudentRegistration().getMusicGroupId();
        Integer signClassId = studentAddDto.getSignClassId();
        List<CourseSchedule> courseScheduleList = new ArrayList<>();
        if (signClassId != null) {
            addStudent(signClassId, userId, nowDate, courseScheduleList);
        }
        Integer mixClassId = studentAddDto.getMixClassId();
        if (mixClassId != null) {
            addStudent(mixClassId, userId, nowDate, courseScheduleList);
        }
        Integer highClassId = studentAddDto.getHighClassId();
        if (highClassId != null) {
            addStudent(highClassId, userId, nowDate, courseScheduleList);
        }
        String snapClassIds = studentAddDto.getSnapClassIds();
        if (StringUtils.isNotEmpty(snapClassIds)) {
            String[] split = snapClassIds.split(",");
            for (int i = 0; i < split.length; i++) {
                addStudent(Integer.parseInt(split[i]), userId, nowDate, courseScheduleList);
            }
        }
        List<CourseScheduleStudentPayment> courseScheduleStudentPayments = new ArrayList<>();
        for (CourseSchedule courseSchedule : courseScheduleList) {
            CourseScheduleStudentPayment courseScheduleStudentPayment = new CourseScheduleStudentPayment();
            courseScheduleStudentPayment.setGroupType(GroupType.MUSIC);
            courseScheduleStudentPayment.setMusicGroupId(musicGroupId);
            courseScheduleStudentPayment.setCourseScheduleId(courseSchedule.getId());
            courseScheduleStudentPayment.setUserId(userId);
            courseScheduleStudentPayment.setExpectPrice(BigDecimal.ZERO);
            courseScheduleStudentPayment.setClassGroupId(signClassId);
            courseScheduleStudentPayment.setCreateTime(nowDate);
            courseScheduleStudentPayment.setUpdateTime(nowDate);
            courseScheduleStudentPayments.add(courseScheduleStudentPayment);
        }

        if (courseScheduleStudentPayments.size() == 0) {
            return;
        }
        courseScheduleStudentPaymentService.batchInsert(courseScheduleStudentPayments);
    }

    @Override
    public List<StudentRegistration> findClassGroupStu(String musicGroupId, Integer classGroupId) {
        //获取所有声部
        List<Subject> subjectList = subjectService.findAll(new HashMap<>());
        List<StudentRegistration> classGroupStuList = studentRegistrationDao.findClassGroupStu(musicGroupId, classGroupId);
        for (StudentRegistration studentRegistration : classGroupStuList) {
            for (Subject subject : subjectList) {
                if (studentRegistration.getActualSubjectId().equals(subject.getId())) {
                    studentRegistration.setSubjectName(subject.getName());
                    break;
                }
            }
        }
        return classGroupStuList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public StudentPaymentOrder updateApplyOrder(StudentPaymentOrder studentPaymentOrder) throws Exception {
        //更新订单状态
        int updateCount = studentPaymentOrderService.update(studentPaymentOrder);
        if (updateCount <= 0) {
            throw new BizException("订单更新失败");
        }
        Date nowDate = new Date();

        //成功报名状态变更
        StudentRegistration studentRegistration = studentRegistrationDao.queryByUserIdAndMusicGroupId(studentPaymentOrder.getUserId(), studentPaymentOrder.getMusicGroupId());
        MusicGroupSubjectPlan musicOneSubjectClassPlan = musicGroupSubjectPlanService.getMusicOneSubjectClassPlan(studentRegistration.getMusicGroupId(), studentRegistration.getActualSubjectId());

        if (studentPaymentOrder.getStatus().equals(DealStatusEnum.SUCCESS)) {
            if (studentRegistration.getTemporaryCourseFee() != null) {
                studentRegistration.setTemporaryCourseFee(new BigDecimal(0));
            }
            studentRegistration.setPaymentStatus(PaymentStatusEnum.YES);
            studentRegistration.setMusicGroupStatus(ClassGroupStudentStatusEnum.NORMAL);
            studentRegistrationDao.update(studentRegistration);

            //进行中乐团加入学生缴费
            if (studentRegistration.getTemporaryCourseFee() != null) {
                //关闭待支付订单
                StudentPaymentOrder waitPayOrder = studentPaymentOrderService.findMusicGroupApplyOrderByStatus(studentRegistration.getUserId(), studentRegistration.getMusicGroupId(), DealStatusEnum.WAIT_PAY);
                waitPayOrder.setStatus(DealStatusEnum.CLOSE);
                studentPaymentOrderService.update(waitPayOrder);
                MusicGroupStudentFee musicGroupStudentFee = musicGroupStudentFeeDao.findByUser(studentRegistration.getUserId(), studentRegistration.getMusicGroupId());
                musicGroupStudentFee.setLatestPaidTime(nowDate);
                musicGroupStudentFee.setPaymentStatus(MusicGroupStudentFee.PaymentStatus.PAID_COMPLETED);
                musicGroupStudentFee.setTemporaryCourseFee(BigDecimal.ZERO);
                musicGroupStudentFee.setUpdateTime(nowDate);
                musicGroupStudentFeeDao.update(musicGroupStudentFee);
                //添加学生
                //classGroupStudentMapperService.addStudents(waitPayOrder.getClassGroupId(), studentRegistration.getUserId().toString(), GroupType.MUSIC);
            }

            //插入交易明细
            BigDecimal amount = studentPaymentOrder.getActualAmount();
            SysUserCashAccount cashAccount = sysUserCashAccountService.get(studentRegistration.getUserId());
            //充值
            SysUserCashAccountDetail rechargeDetail = new SysUserCashAccountDetail();
            rechargeDetail.setAmount(amount);
            rechargeDetail.setBalance(cashAccount.getBalance().add(amount));
            rechargeDetail.setComment("缴费前充值");
            rechargeDetail.setCreateTime(nowDate);
            rechargeDetail.setStatus(DealStatusEnum.SUCCESS);
            rechargeDetail.setTransNo(studentPaymentOrder.getTransNo());
            rechargeDetail.setType(PlatformCashAccountDetailTypeEnum.RECHARGE);
            rechargeDetail.setUpdateTime(nowDate);
            rechargeDetail.setUserId(studentRegistration.getUserId());
            rechargeDetail.setChannel(studentPaymentOrder.getPaymentChannel());
            rechargeDetail.setComAmount(studentPaymentOrder.getComAmount());
            rechargeDetail.setPerAmount(studentPaymentOrder.getPerAmount());
            sysUserCashAccountDetailService.insert(rechargeDetail);

            //缴费
            SysUserCashAccountDetail paymentDetail = new SysUserCashAccountDetail();
            paymentDetail.setAmount(amount.negate());
            paymentDetail.setBalance(cashAccount.getBalance());
            paymentDetail.setComment("报名缴费");
            paymentDetail.setCreateTime(nowDate);
            paymentDetail.setStatus(DealStatusEnum.SUCCESS);
            paymentDetail.setTransNo(studentPaymentOrder.getTransNo());
            paymentDetail.setType(PlatformCashAccountDetailTypeEnum.PAY_FEE);
            paymentDetail.setUpdateTime(nowDate);
            paymentDetail.setUserId(studentRegistration.getUserId());
            rechargeDetail.setChannel(studentPaymentOrder.getPaymentChannel());
            if (studentPaymentOrder.getComAmount() != null) {
                rechargeDetail.setComAmount(studentPaymentOrder.getComAmount().negate());
                rechargeDetail.setPerAmount(studentPaymentOrder.getPerAmount().negate());
            }

            sysUserCashAccountDetailService.insert(paymentDetail);

            MusicGroup musicGroup = musicGroupDao.get(studentRegistration.getMusicGroupId());
            //三方乐团不发送缴费通知
            if (musicGroup.getOwnershipType() != null && musicGroup.getOwnershipType() == CooperationOrgan.OwnershipType.OWN) {
                //发送缴费成功通知（短信 + push）
                Map<Integer, String> map = new HashMap<>(1);
                map.put(studentPaymentOrder.getUserId(), studentRegistration.getParentsPhone());
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI,
                        MessageTypeEnum.STUDENT_SMS_PAYMENT_SUCCESS, map, null, 0, "1", "",
                        studentRegistration.getParentsName(), studentPaymentOrder.getActualAmount());
                //push
                Map<Integer, String> map1 = new HashMap<>(1);
                map1.put(studentPaymentOrder.getUserId(), studentPaymentOrder.getUserId().toString());
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG,
                        MessageTypeEnum.STUDENT_SMS_PAYMENT_SUCCESS, map1, null, 0, "1", "STUDENT",
                        studentRegistration.getParentsName(), studentPaymentOrder.getActualAmount());
            }
            try {
                //生成课程协议
                contractService.transferMusicGroupCoursesContract(studentRegistration.getUserId(), studentRegistration.getMusicGroupId());
                // 商品协议(租赁时候有)
                StudentPaymentOrderDetail applyOrderMusical = studentPaymentOrderDetailService.findApplyOrderMusical(studentPaymentOrder.getId());
                if (applyOrderMusical != null && applyOrderMusical.getKitGroupPurchaseType() != null && applyOrderMusical.getKitGroupPurchaseType().equals(KitGroupPurchaseTypeEnum.LEASE)) {
                        contractService.transferGoodsContract(studentPaymentOrder.getUserId(), studentPaymentOrder.getMusicGroupId(),
                                applyOrderMusical.getGoodsIdList(), applyOrderMusical.getKitGroupPurchaseType());
                }
            } catch (Exception e) {
                LOGGER.error("协议生成失败", e);
            }
        }


        //减去缴费人数
        if (studentPaymentOrder.getStatus().equals(DealStatusEnum.FAILED)) {
            if (studentPaymentOrder.getActualAmount().compareTo(BigDecimal.ZERO) > 0) {
                musicOneSubjectClassPlan.setPaidStudentNum(musicOneSubjectClassPlan.getPaidStudentNum() - 1);
                updateCount = musicGroupSubjectPlanService.update(musicOneSubjectClassPlan);
                if (updateCount <= 0) {
                    throw new BizException("减去缴费人数失败");
                }
            }
            if (studentPaymentOrder.getBalancePaymentAmount() != null && studentPaymentOrder.getBalancePaymentAmount().compareTo(BigDecimal.ZERO) > 0) {
                sysUserCashAccountService.updateBalance(studentPaymentOrder.getUserId(), studentPaymentOrder.getBalancePaymentAmount(), PlatformCashAccountDetailTypeEnum.REFUNDS, "报名缴费失败");
            }
            MusicGroup musicGroup = musicGroupDao.get(studentRegistration.getMusicGroupId());
            //三方乐团不发送缴费通知
            if (musicGroup.getOwnershipType() != null && musicGroup.getOwnershipType() == CooperationOrgan.OwnershipType.OWN) {
                String studentApplyUrl = sysConfigDao.findConfigValue(SysConfigService.STUDENT_APPLY_URL) + studentRegistration.getMusicGroupId();
                Map<Integer, String> map = new HashMap<>(1);
                map.put(studentPaymentOrder.getUserId(), studentRegistration.getParentsPhone());
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI,
                        MessageTypeEnum.STUDENT_SMS_PAYMENT_FAILED, map, null, 0, "1", "",
                        HttpUtil.getSortUrl(studentApplyUrl));
            }
//            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG,
//                    MessageTypeEnum.STUDENT_SMS_PAYMENT_FAILED,map,null,0,"1",
//                    studentApplyUrl);
        }

        return studentPaymentOrder;
    }

    @Override
    public List<StudentRegistration> findMusicGroupNoClassGroupStudent(String musicGroupId, Integer actualSubjectId) {
        //获取所有声部
        List<Subject> subjectList = subjectService.findAll(new HashMap<>());
        List<StudentRegistration> classGroupStuList = studentRegistrationDao.findMusicGroupNoClassGroupStudent(musicGroupId, actualSubjectId);
        for (StudentRegistration studentRegistration : classGroupStuList) {
            for (Subject subject : subjectList) {
                if (studentRegistration.getActualSubjectId().equals(subject.getId())) {
                    studentRegistration.setSubjectName(subject.getName());
                    break;
                }
            }
        }
        return classGroupStuList;
    }

    @Override
    public List<StudentRegistration> findStudentListByIdList(List<Long> idList) {
        return studentRegistrationDao.findStudentListByIdList(idList);
    }

    @Override
    public int batchInsert(List<StudentRegistration> studentRegistrationList) {
        return studentRegistrationDao.batchInsert(studentRegistrationList);
    }

    @Override
    public StudentRegistration findStudentByClassGroupIdAndUserId(Integer userId, Integer classGroupId) {
        return studentRegistrationDao.findStudentByClassGroupIdAndUserId(userId, classGroupId);
    }

    @Override
    public List<StudentRegistration> findMusicGroupStudent(String musicGroupId, Integer actualSubjectId) {
        List<Subject> subjectList = subjectService.findAll(new HashMap<>());
        List<StudentRegistration> musicGroupStudents = studentRegistrationDao.findMusicGroupStudent(musicGroupId, actualSubjectId);
        for (StudentRegistration musicGroupStudent : musicGroupStudents) {
            for (Subject subject : subjectList) {
                if (musicGroupStudent.getActualSubjectId().equals(subject.getId())) {
                    musicGroupStudent.setSubjectName(subject.getName());
                }
            }
        }
        return musicGroupStudents;
    }

    @Override
    public List<StudentRegistration> findStudentListByUserIdList(String musicGroupId, List<Integer> userIdList) {
        return studentRegistrationDao.findStudentListByUserIdList(musicGroupId, userIdList);
    }

    @Override
    public int openPayment(String ids) throws IOException {
        if (StringUtils.isNotEmpty(ids)) {
            //发送短信通知
            List<Long> list = JSONObject.parseArray(JSONObject.toJSONString(Arrays.asList(ids.split(","))), Long.class);
            MusicGroup musicGroup = musicGroupDao.findMusicGroup(list);
            //三方乐团不发送缴费通知
            if (musicGroup.getOwnershipType() != null && musicGroup.getOwnershipType() == CooperationOrgan.OwnershipType.OWN) {
                List<StudentRegistration> registrations = studentRegistrationDao.findStudentListByIdList(list);
                String configValue = sysConfigDao.findConfigValue(SysConfigService.STUDENT_APPLY_URL);
                HashMap<Integer, String> receivers = new HashMap<>(registrations.size());
                registrations.forEach(e -> {
                    receivers.put(e.getUserId(), e.getParentsPhone());
                });
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI,
                        MessageTypeEnum.SMS_STUDENT_OPEN_PAYMENT, receivers, null, 0, null, "",
                        musicGroup.getName(), HttpUtil.getSortUrl(configValue + musicGroup.getId()));
            }
        }
        return studentRegistrationDao.batchOpenPay(ids, PaymentStatusEnum.OPEN);
    }

    @Override
    public List<StudentRegistration> queryStudentByMusicGroupId(String musicGroupId) {
        return studentRegistrationDao.queryStudentByMusicGroupId(musicGroupId);
    }

    @Override
    public List<Map<Integer, String>> findParentId(String parentPhones) {
        return studentRegistrationDao.findParentId(parentPhones);
    }

    @Override
    @Transactional
    public StudentRegistration updateStudent(StudentRegistration studentRegistration) {
        StudentRegistration student = get(studentRegistration.getId());
        if (student == null) {
            throw new BizException("用户注册信息不存在，无法更新");
        }
        student.setParentsName(studentRegistration.getParentsName());
        student.setIdCardNo(studentRegistration.getIdCardNo());
        update(student);
        studentRegistrationDao.updateUser(student.getUserId(), student.getParentsName(), student.getIdCardNo());
        // 添加用户电子签章账户
        contractService.register(student.getUserId(), student.getParentsName(), student.getIdCardNo(), student.getParentsPhone());
        return student;
    }

    @Override
    public StudentRegistration queryUserByPhone(String mobile) {
        return studentRegistrationDao.queryUserByPhone(mobile);
    }

    @Override
    public List<StudentRegistration> findMusicGroupStuNoClassType(String musicGroupId, ClassGroupTypeEnum type, Integer subjectId) {
        List<StudentRegistration> students = studentRegistrationDao.findMusicGroupStudent(musicGroupId, subjectId);
        List<Subject> subjectList = subjectService.findAll(new HashMap<>());

        List<ClassGroupStudentMapper> classGroupStudentMappers = classGroupStudentMapperDao.findMusicGroupClassGroupByType(musicGroupId, type);
        Iterator<StudentRegistration> iterator = students.iterator();
        while (iterator.hasNext()) {
            StudentRegistration student = iterator.next();
            for (Subject subject : subjectList) {
                if (student.getActualSubjectId().equals(subject.getId())) {
                    student.setSubjectName(subject.getName());
                    break;
                }
            }
            if (type.equals(ClassGroupTypeEnum.SNAP)) continue;
            for (ClassGroupStudentMapper classGroupStudentMapper : classGroupStudentMappers) {
                if (student.getUserId().equals(classGroupStudentMapper.getUserId())) {
                    iterator.remove();
                    break;
                }
            }
        }
        return students;
    }
}
